@@ -20,15 +20,16 @@ import functools
 import logging
 import os
 import re
+import types
 import typing as t
 from abc import ABC, ABCMeta, abstractmethod
 
 import bentoml
 import inflection
+from bentoml.types import ModelSignature, ModelSignatureDict
 
 import openllm
 
-from ._configuration import ModelSignature
 from .exceptions import ForbiddenAttributeError, OpenLLMException
 from .utils import ENV_VARS_TRUE_VALUES, LazyLoader, bentoml_cattr
 
@@ -37,13 +38,23 @@ if t.TYPE_CHECKING:
     import transformers
     from bentoml._internal.runner.strategy import Strategy
 
-    from ._types import LLMModel, LLMTokenizer, ModelSignatureDict
+    from ._types import LLMModel, LLMTokenizer
+
+    class LLMRunner(bentoml.Runner):
+        llm: openllm.LLM
+        llm_type: str
+        identifying_params: dict[str, t.Any]
+
+        def __call__(self, *args: t.Any, **attrs: t.Any) -> t.Any:
+            ...
+
 else:
-    ModelSignatureDict = dict
     transformers = LazyLoader("transformers", globals(), "transformers")
 
 logger = logging.getLogger(__name__)
 
+_object_setattr = object.__setattr__
+
 # NOTE: `1-2` -> text-generation and text2text-generation
 FRAMEWORK_TO_AUTOCLASS_MAPPING = {
     "pt": ("AutoModelForCausalLM", "AutoModelForSeq2SeqLM"),
@@ -194,25 +205,36 @@ class LLMInterface(ABC):
     """The default PyPI requirements needed to run this given LLM. By default, we will depend on 
     bentoml, torch, transformers."""
 
-    def preprocess_parameters(self, prompt: str, **generation_kwds: t.Any) -> tuple[str, dict[str, t.Any]]:
-        """This handler will preprocess given prompt into the correct inputs with all
-        filled templates that the model accepts. It will returns a tuple of expected LLM prompt type and and dictionary
-        of expected arguments to be passed into model.generate.
-
-        You can customize the generation attributes for generate here. By default, it is a simple echo.
-
-        NOTE: this will be used from the client side.
-        """
-        return prompt, generation_kwds
-
     @abstractmethod
     def generate(self, prompt: str, **preprocess_generate_kwds: t.Any) -> t.Any:
-        """The main function implementation for generating from given prompt. It takes the outputs
-        of preprocess_generate, and then those can be pass to 'model.generate'.
+        """The main function implementation for generating from given prompt.  It takes the prompt
+        and the generation_kwargs from 'self.sanitize_parameters' and then
+        pass it to 'self.model.generate'.
         """
         raise NotImplementedError
 
-    def postprocess_parameters(self, prompt: str, generation_result: t.Any, **attrs: t.Any) -> t.Any:
+    def generate_iterator(self, prompt: str, **attrs: t.Any) -> t.Iterator[t.Any]:
+        """An iterator version of generate function."""
+        raise NotImplementedError(
+            "Currently generate_iterator requires SSE (Server-side events) support, which is not yet implemented."
+        )
+
+    def sanitize_parameters(self, prompt: str, **attrs: t.Any) -> tuple[str, dict[str, t.Any], dict[str, t.Any]]:
+        """This handler will sanitize all attrs and setup prompt text.
+
+        It takes a prompt that is given by the user, attrs that can be parsed with the prompt.
+
+        NOTE: the attrs should also handle the following default attributes from all LLMConfig:
+        - use_default_prompt_template
+
+        Returns a tuple of three items:
+        - The processed prompt text depending on `use_default_prompt_template`
+        - The attributes dictionary that can be passed into LLMConfig to generate a GenerationConfig
+        - The attributes dictionary that will be passed into `self.postprocess_generate`.
+        """
+        return prompt, attrs, attrs
+
+    def postprocess_generate(self, prompt: str, generation_result: t.Any, **attrs: t.Any) -> t.Any:
         """This handler will postprocess generation results from LLM.generate and
         then output nicely formatted results (if the LLM decide to do so.)
 
@@ -222,12 +244,6 @@ class LLMInterface(ABC):
         """
         return generation_result
 
-    def generate_iterator(self, prompt: str, **attrs: t.Any) -> t.Iterator[t.Any]:
-        """An iterator version of generate function."""
-        raise NotImplementedError(
-            "Currently generate_iterator requires SSE (Server-side events) support, which is not yet implemented."
-        )
-
     def llm_post_init(self):
         """This function can be implemented if you need to initialized any additional variables that doesn't
         concern OpenLLM internals.
@@ -270,14 +286,11 @@ class LLMMetaclass(ABCMeta):
             if cls_name.startswith("Flax"):
                 implementation = "flax"
                 prefix_class_name_config = cls_name[4:]
-                namespace["__annotations__"].update({"model": "transformers.FlaxPreTrainedModel"})
             elif cls_name.startswith("TF"):
                 implementation = "tf"
                 prefix_class_name_config = cls_name[2:]
-                namespace["__annotations__"].update({"model": "transformers.TFPreTrainedModel"})
             else:
                 implementation = "pt"
-                namespace["__annotations__"].update({"model": "transformers.PreTrainedModel"})
             namespace["__llm_implementation__"] = implementation
 
             # NOTE: setup config class branch
@@ -291,6 +304,9 @@ class LLMMetaclass(ABCMeta):
 
             config_class: type[openllm.LLMConfig] = namespace["config_class"]
 
+            # NOTE: update the annotations for self.config
+            namespace["__annotations__"]["config"] = t.get_type_hints(config_class)
+
             for key in ("__openllm_start_name__", "__openllm_requires_gpu__"):
                 namespace[key] = getattr(config_class, key)
 
@@ -337,10 +353,6 @@ class LLM(LLMInterface, metaclass=LLMMetaclass):
         __openllm_requires_gpu__: bool
         __openllm_post_init__: t.Callable[[t.Self], None] | None
 
-        # NOTE: the following will be populated by __init__
-        config: openllm.LLMConfig
-        load_in_mha: bool
-
     # NOTE: the following is the similar interface to HuggingFace pretrained protocol.
 
     @classmethod
@@ -714,10 +726,18 @@ class LLM(LLMInterface, metaclass=LLMMetaclass):
             llm_type: str
             identifying_params: dict[str, t.Any]
 
-            def __init_subclass__(cls, **attrs: t.Any):
-                cls.llm_type = self.llm_type
-                cls.identifying_params = self.identifying_params
-                super().__init_subclass__(**attrs)
+            def __init_subclass__(cls, llm_type: str, identifying_params: dict[str, t.Any], **_: t.Any):
+                cls.llm_type = llm_type
+                cls.identifying_params = identifying_params
+
+            @bentoml.Runnable.method(
+                batchable=generate_sig.batchable,
+                batch_dim=generate_sig.batch_dim,
+                input_spec=generate_sig.input_spec,
+                output_spec=generate_sig.output_spec,
+            )
+            def __call__(__self, prompt: str, **attrs: t.Any) -> list[t.Any]:
+                return self.generate(prompt, **attrs)
 
             @bentoml.Runnable.method(
                 batchable=generate_sig.batchable,
@@ -737,16 +757,51 @@ class LLM(LLMInterface, metaclass=LLMMetaclass):
             def generate_iterator(__self, prompt: str, **attrs: t.Any) -> t.Iterator[t.Any]:
                 yield self.generate_iterator(prompt, **attrs)
 
-        if self.__openllm_requires_gpu__:
-            _supported_resources = ("nvidia.com/gpu",)
-        else:
-            _supported_resources = ("nvidia.com/gpu", "cpu")
+        def _wrapped_generate_run(__self: bentoml.Runner, *args: t.Any, **kwargs: t.Any) -> t.Any:
+            """Wrapper for runner.generate.run() to handle the prompt and postprocessing.
 
-        return bentoml.Runner(
-            type(
+            This will be used for LangChain API.
+
+            Usage:
+            ```python
+            runner = openllm.Runner("dolly-v2", init_local=True)
+            runner("What is the meaning of life?")
+            ```
+            """
+            if len(args) > 1:
+                raise RuntimeError("Only one positional argument is allowed for generate()")
+            prompt = args[0] if len(args) == 1 else kwargs.pop("prompt", "")
+
+            prompt, generate_kwargs, postprocess_kwargs = self.sanitize_parameters(prompt, **kwargs)
+            generated_result = __self.generate.run(prompt, **generate_kwargs)
+            return self.postprocess_generate(prompt, generated_result, **postprocess_kwargs)
+
+        # NOTE: returning the two langchain API's to the runner
+        return types.new_class(
+            inflection.camelize(self.config.__openllm_model_name__) + "Runner",
+            (bentoml.Runner,),
+            exec_body=lambda ns: ns.update(
+                {
+                    "llm_type": self.llm_type,
+                    "identifying_params": self.identifying_params,
+                    "llm": self,  # NOTE: self reference to LLM
+                    "config": self.config,
+                    "__call__": _wrapped_generate_run,
+                    "__module__": f"openllm.models.{self.config.__openllm_model_name__}",
+                    "__doc__": self.config.__openllm_env__.start_docstring,
+                }
+            ),
+        )(
+            types.new_class(
                 inflection.camelize(self.config.__openllm_model_name__) + "Runnable",
                 (_Runnable,),
-                {"SUPPORTED_RESOURCES": _supported_resources},
+                {
+                    "SUPPORTED_RESOURCES": ("nvidia.com/gpu", "cpu")
+                    if self.__openllm_requires_gpu__
+                    else ("nvidia.com/gpu",),
+                    "llm_type": self.llm_type,
+                    "identifying_params": self.identifying_params,
+                },
             ),
             name=name,
             models=models,
@@ -757,6 +812,21 @@ class LLM(LLMInterface, metaclass=LLMMetaclass):
             scheduling_strategy=scheduling_strategy,
         )
 
+    def predict(self, prompt: str, **attrs: t.Any) -> t.Any:
+        """The scikit-compatible API for self(...)"""
+        return self.__call__(prompt, **attrs)
+
+    def __call__(self, prompt: str, **attrs: t.Any) -> t.Any:
+        """Returns the generation result and format the result.
+
+        First, it runs `self.sanitize_parameters` to sanitize the parameters.
+        The the sanitized prompt and kwargs will be pass into self.generate.
+        Finally, run self.postprocess_generate to postprocess the generated result.
+        """
+        prompt, generate_kwargs, postprocess_kwargs = self.sanitize_parameters(prompt, **attrs)
+        generated_result = self.generate(prompt, **generate_kwargs)
+        return self.postprocess_generate(prompt, generated_result, **postprocess_kwargs)
+
 
 def Runner(start_name: str, **attrs: t.Any) -> bentoml.Runner:
     """Create a Runner for given LLM. For a list of currently supported LLM, check out 'openllm models'
@@ -777,6 +847,6 @@ def Runner(start_name: str, **attrs: t.Any) -> bentoml.Runner:
         runner = openllm.AutoLLM.create_runner(start_name, **attrs)
 
     if init_local:
-        runner.init_local()
+        runner.init_local(quiet=True)
 
     return runner
