@@ -15,6 +15,7 @@
 Types definition for OpenLLM.
 
 Note that this module SHOULD NOT BE IMPORTED DURING RUNTIME, as this serve only for typing purposes.
+It will raises a RuntimeError if this is imported eagerly.
 """
 from __future__ import annotations
 
@@ -23,11 +24,32 @@ import typing as t
 if not t.TYPE_CHECKING:
     raise RuntimeError(f"{__name__} should not be imported during runtime")
 
+import click
+
+P = t.ParamSpec("P")
+O_co = t.TypeVar("O_co", covariant=True)
+
 import transformers
-from bentoml._internal.models.model import ModelSignatureDict as ModelSignatureDict
 from bentoml._internal.models.model import ModelSignaturesType as ModelSignaturesType
 
 LLMModel = transformers.PreTrainedModel | transformers.TFPreTrainedModel | transformers.FlaxPreTrainedModel
 LLMTokenizer = (
     transformers.PreTrainedTokenizer | transformers.PreTrainedTokenizerFast | transformers.PreTrainedTokenizerBase
 )
+
+
+class ClickFunctionWrapper(t.Protocol[P, O_co]):
+    __name__: str
+    __click_params__: list[click.Option]
+
+    def __call__(self, *args: P.args, **kwargs: P.kwargs) -> O_co:
+        ...
+
+
+# F is a t.Callable[P, O_co] with compatible to ClickFunctionWrapper
+class F(t.Generic[P, O_co]):
+    __name__: str
+    __click_params__: list[click.Option]
+
+    def __call__(self, *args: P.args, **kwargs: P.kwargs) -> O_co:
+        ...
